# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################
"""Test for the `Parser` base class."""
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from aiida import orm
from aiida.backends.testbase import AiidaTestCase
from aiida.common import LinkType
from aiida.engine import CalcJob
from aiida.parsers import Parser
from aiida.plugins import CalculationFactory, ParserFactory

ArithmeticAddCalculation = CalculationFactory('arithmetic.add')  # pylint: disable=invalid-name
ArithmeticAddParser = ParserFactory('arithmetic.add')  # pylint: disable=invalid-name


class CustomCalcJob(CalcJob):
    """`CalcJob` implementation with output namespace and additional output node that should be passed to parser."""

    @classmethod
    def define(cls, spec):
        super(CustomCalcJob, cls).define(spec)
        spec.input('inp', valid_type=orm.Data)
        spec.output('output', pass_to_parser=True)
        spec.output_namespace('out.space', dynamic=True)

    def prepare_for_submission(self):  # pylint: disable=arguments-differ
        pass


class TestParser(AiidaTestCase):
    """Test backend entities and their collections"""

    def test_abstract_parse_method(self):
        """Verify that trying to instantiate base class will raise `TypeError` because of abstract `parse` method."""
        with self.assertRaises(TypeError):
            Parser()  # pylint: disable=abstract-class-instantiated,no-value-for-parameter

    def test_parser_retrieved(self):
        """Verify that the `retrieved` property returns the retrieved `FolderData` node."""
        node = orm.CalcJobNode(computer=self.computer, process_type=ArithmeticAddCalculation.build_process_type())
        node.set_option('resources', {'num_machines': 1, 'num_mpiprocs_per_machine': 1})
        node.set_option('max_wallclock_seconds', 1800)
        node.store()

        retrieved = orm.FolderData().store()
        retrieved.add_incoming(node, link_type=LinkType.CREATE, link_label='retrieved')

        parser = ArithmeticAddParser(node)
        self.assertEqual(parser.node.uuid, node.uuid)
        self.assertEqual(parser.retrieved.uuid, retrieved.uuid)

    def test_parser_exit_codes(self):
        """Ensure that exit codes from the `CalcJob` can be retrieved through the parser instance."""
        node = orm.CalcJobNode(computer=self.computer, process_type=ArithmeticAddCalculation.build_process_type())
        parser = ArithmeticAddParser(node)
        self.assertEqual(parser.exit_codes, ArithmeticAddCalculation.spec().exit_codes)

    def test_parser_get_outputs_for_parsing(self):
        """Make sure that the `get_output_for_parsing` method returns the correct output nodes."""
        ArithmeticAddCalculation.define = CustomCalcJob.define
        node = orm.CalcJobNode(computer=self.computer, process_type=CustomCalcJob.build_process_type())
        node.set_option('resources', {'num_machines': 1, 'num_mpiprocs_per_machine': 1})
        node.set_option('max_wallclock_seconds', 1800)
        node.store()

        retrieved = orm.FolderData().store()
        retrieved.add_incoming(node, link_type=LinkType.CREATE, link_label='retrieved')

        output = orm.Data().store()
        output.add_incoming(node, link_type=LinkType.CREATE, link_label='output')

        parser = ArithmeticAddParser(node)
        outputs_for_parsing = parser.get_outputs_for_parsing()
        self.assertIn('retrieved', outputs_for_parsing)
        self.assertEqual(outputs_for_parsing['retrieved'].uuid, retrieved.uuid)
        self.assertIn('output', outputs_for_parsing)
        self.assertEqual(outputs_for_parsing['output'].uuid, output.uuid)



# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import copy
import re

from six.moves import zip
import numpy as np

from aiida.common.constants import elements
from aiida.orm.nodes.data.structure import Kind, Site, StructureData
from aiida.engine import calcfunction

__all__ = ('structure_to_spglib_tuple', 'spglib_tuple_to_structure')


@calcfunction
def _get_cif_ase_inline(struct, parameters):
    """
    Creates :py:class:`aiida.orm.nodes.data.cif.CifData` using ASE.

    .. note:: requires ASE module.
    """
    from aiida.orm import CifData

    kwargs = {}
    if parameters is not None:
        kwargs = parameters.get_dict()
    cif = CifData(ase=struct.get_ase(**kwargs))
    formula = struct.get_formula(mode='hill', separator=' ')
    for i in cif.values.keys():
        cif.values[i]['_symmetry_space_group_name_H-M'] = 'P 1'
        cif.values[i]['_symmetry_space_group_name_Hall'] = 'P 1'
        cif.values[i]['_symmetry_Int_Tables_number'] = 1
        cif.values[i]['_cell_formula_units_Z'] = 1
        cif.values[i]['_chemical_formula_sum'] = formula
    return {'cif': cif}


def structure_to_spglib_tuple(structure):
    """
    Convert an AiiDA structure to a tuple of the format (cell, scaled_positions, element_numbers).

    :param structure: the AiiDA structure
    :return: (structure_tuple, kind_info, kinds) where structure_tuple
        is a tuple of format (cell, scaled_positions, element_numbers);
        kind_info is a dictionary mapping the kind_names to
        the numbers used in element_numbers. When possible, it uses
        the Z number of the element, otherwise it uses numbers > 1000;
        kinds is a list of the kinds of the structure.
    """
    def get_new_number(the_list, start_from):
        """
        Get the first integer >= start_from not yet in the list
        """
        retval = start_from
        comp_list = sorted(_ for _ in the_list if _ >= start_from)

        current_pos = 0
        found = False
        while not found:
            if len(comp_list) <= current_pos:
                return retval
            if retval == comp_list[current_pos]:
                current_pos += 1
                retval += 1
            else:
                found = True
                return retval

    Z = {v['symbol']: k for k, v in elements.items()}

    cell = np.array(structure.cell)
    abs_pos = np.array([_.position for _ in structure.sites])
    rel_pos = np.dot(abs_pos, np.linalg.inv(cell))
    kinds = {k.name: k for k in structure.kinds}

    kind_numbers = {}
    for kind in structure.kinds:
        if len(kind.symbols) == 1:
            realnumber = Z[kind.symbols[0]]
            if realnumber in kind_numbers.values():
                number = get_new_number(
                    list(kind_numbers.values()), start_from=realnumber * 1000)
            else:
                number = realnumber
            kind_numbers[kind.name] = number
        else:
            number = get_new_number(list(kind_numbers.values()), start_from=200000)
            kind_numbers[kind.name] = number

    numbers = [kind_numbers[s.kind_name] for s in structure.sites]

    return ((cell, rel_pos, numbers), kind_numbers, list(structure.kinds))


def spglib_tuple_to_structure(structure_tuple, kind_info=None, kinds=None):
    """
    Convert a tuple of the format (cell, scaled_positions, element_numbers) to an AiiDA structure.

    Unless the element_numbers are identical to the Z number of the atoms,
    you should pass both kind_info and kinds, with the same format as returned
    by get_tuple_from_aiida_structure.

    :param structure_tuple: the structure in format (structure_tuple, kind_info)
    :param kind_info: a dictionary mapping the kind_names to
       the numbers used in element_numbers. If not provided, assumes {element_name: element_Z}
    :param kinds: a list of the kinds of the structure.
    """
    if kind_info is None and kinds is not None:
        raise ValueError("If you pass kind_info, you should also pass kinds")
    if kinds is None and kind_info is not None:
        raise ValueError("If you pass kinds, you should also pass kind_info")

    Z = {v['symbol']: k for k, v in elements.items()}
    cell, rel_pos, numbers = structure_tuple
    if kind_info:
        _kind_info = copy.copy(kind_info)
        _kinds = copy.copy(kinds)
    else:
        try:
            # For each site
            symbols = [elements[num]['symbol'] for num in numbers]
        except KeyError as exc:
            raise ValueError("You did not pass kind_info, but at least one number "
                             "is not a valid Z number: {}".format(exc.args[0]))

        _kind_info = {elements[num]['symbol']: num for num in set(numbers)}
        # Get the default kinds
        _kinds = [Kind(symbols=sym) for sym in set(symbols)]

    _kinds_dict = {k.name: k for k in _kinds}
    # Now I will use in any case _kinds and _kind_info
    if len(_kind_info) != len(set(_kind_info.values())):
        raise ValueError(
            "There is at least a number repeated twice in kind_info!")
    # Invert the mapping
    mapping_num_kindname = {v: k for k, v in _kind_info.items()}
    # Create the actual mapping
    try:
        mapping_to_kinds = {num: _kinds_dict[kindname] for num, kindname
                            in mapping_num_kindname.items()}
    except KeyError as exc:
        raise ValueError(
            "Unable to find '{}' in the kinds list".format(exc.args[0]))

    try:
        site_kinds = [mapping_to_kinds[num] for num in numbers]
    except KeyError as exc:
        raise ValueError("Unable to find kind in kind_info for number {}".format(exc.args[0]))

    structure = StructureData(cell=cell)
    for k in _kinds:
        structure.append_kind(k)
    abs_pos = np.dot(rel_pos, cell)
    if len(abs_pos) != len(site_kinds):
        raise ValueError("The length of the positions array is different from the "
                         "length of the element numbers")

    for kind, pos in zip(site_kinds, abs_pos):
        structure.append_site(Site(kind_name=kind.name, position=pos))

    return structure


def xyz_parser_iterator(xyz_string):
    """
    Yields a tuple `(natoms, comment, atomiter)`for each frame
    in a XYZ file where `atomiter` is an iterator yielding a
    nested tuple `(symbol, (x, y, z))` for each entry.

    :param xyz_string: a string containing XYZ-structured text
    """

    class BlockIterator(object):
        """
        An iterator for wrapping the iterator returned by `match.finditer`
        to extract the required fields directly from the match object
        """

        def __init__(self, it, natoms):
            self._it = it
            self._natoms = natoms
            self._catom = 0

        def __iter__(self):
            return self

        def __next__(self):  # pylint: disable=missing-docstring
            try:
                match = next(self._it)
            except StopIteration:
                # if we reached the number of atoms declared, everything is well
                # and we re-raise the StopIteration exception
                if self._catom == self._natoms:
                    raise
                else:
                    # otherwise we got too less entries
                    raise TypeError("Number of atom entries ({}) is smaller than the number of atoms ({})".format(
                        self._catom, self._natoms))

            self._catom += 1

            if self._catom > self._natoms:
                raise TypeError("Number of atom entries ({}) is larger than the number of atoms ({})".format(
                    self._catom, self._natoms))

            return (match.group('sym'), (float(match.group('x')), float(match.group('y')), float(match.group('z'))))

        def next(self):
            """
            The iterator method expected by python 2.x,
            implemented as python 3.x style method.
            """
            return self.__next__()

    pos_regex = re.compile(
        r"""
^                                                                             # Linestart
[ \t]*                                                                        # Optional white space
(?P<sym>[A-Za-z]+[A-Za-z0-9]*)\s+                                             # get the symbol
(?P<x> [\+\-]?  ( \d*[\.]\d+  | \d+[\.]?\d* )  ([Ee][\+\-]?\d+)? ) [ \t]+     # Get x
(?P<y> [\+\-]?  ( \d*[\.]\d+  | \d+[\.]?\d* )  ([Ee][\+\-]?\d+)? ) [ \t]+     # Get y
(?P<z> [\+\-]?  ( \d*[\.]\d+  | \d+[\.]?\d* )  ([Ee][\+\-]?\d+)? )            # Get z
""", re.X | re.M)
    pos_block_regex = re.compile(
        r"""
                                                            # First line contains an integer
                                                            # and only an integer: the number of atoms
^[ \t]* (?P<natoms> [0-9]+) [ \t]*[\n]                      # End first line
(?P<comment>.*) [\n]                                        # The second line is a comment
(?P<positions>                                              # This is the block of positions
    (
        (
            \s*                                             # White space in front of the element spec is ok
            (
                [A-Za-z]+[A-Za-z0-9]*                       # Element spec
                (
                    \s+                                     # White space in front of the number
                    [\+\-]?                                 # Plus or minus in front of the number (optional)
                    (
                        (
                            \d*                             # optional decimal in the beginning .0001 is ok, for example
                            [\.]                            # There has to be a dot followed by
                            \d+                             # at least one decimal
                        )
                        |                                   # OR
                        (
                            \d+                             # at least one decimal, followed by
                            [\.]?                           # an optional dot
                            \d*                             # followed by optional decimals
                        )
                    )
                    ([Ee][\+\-]?\d+)?                       # optional exponents E+03, e-05
                ){3}                                        # I expect three float values
                |
                \#                                          # If a line is commented out, that is also ok
            )
            .*                                              # ignore what is after the comment or the position spec
            |                                               # OR
            \s*                                             # A line only containing white space
         )
        [\n]                                                # line break at the end
    )+
)                                                           # A positions block should be one or more lines
                    """, re.X | re.M)

    for block in pos_block_regex.finditer(xyz_string):
        natoms = int(block.group('natoms'))
        yield (natoms, block.group('comment'), BlockIterator(pos_regex.finditer(block.group('positions')), natoms))



# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import
from aiida.backends.testbase import AiidaTestCase
from aiida.common.exceptions import NotExistent
from aiida.orm import Node, Group, Data
from aiida.orm.utils import load_entity, load_code, load_computer, load_group, load_node
from aiida.orm.utils.loaders import NodeEntityLoader


class TestOrmUtils(AiidaTestCase):

    def test_load_entiy(self):
        """Test the functionality of load_entity which is the base function for the other loader functions."""
        entity_loader = NodeEntityLoader

        with self.assertRaises(TypeError):
            load_entity(entity_loader=None)

        # No identifier keyword arguments specified
        with self.assertRaises(ValueError):
            load_entity(entity_loader)

        # More than one identifier keyword arguments specified
        with self.assertRaises(ValueError):
            load_entity(entity_loader, identifier='a', pk=1)

        with self.assertRaises(TypeError):
            load_entity(entity_loader, pk='1')

        with self.assertRaises(TypeError):
            load_entity(entity_loader, uuid=1)

        with self.assertRaises(TypeError):
            load_entity(entity_loader, label=1)

    def test_load_code(self):
        """Test the functionality of load_code."""
        from aiida.orm import Code

        label = 'compy'
        code = Code()
        code.label = label
        code.set_remote_computer_exec((self.computer, '/x.x'))
        code.store()

        # Load through full label
        loaded_code = load_code(code.full_label)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through label
        loaded_code = load_code(code.label)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through uuid
        loaded_code = load_code(code.uuid)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through pk
        loaded_code = load_code(code.pk)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through full label explicitly
        loaded_code = load_code(label=code.full_label)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through label explicitly
        loaded_code = load_code(label=code.label)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through uuid explicitly
        loaded_code = load_code(uuid=code.uuid)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through pk explicitly
        loaded_code = load_code(pk=code.pk)
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through partial uuid without a dash
        loaded_code = load_code(uuid=code.uuid[:8])
        self.assertEquals(loaded_code.uuid, code.uuid)

        # Load through partial uuid including a dash
        loaded_code = load_code(uuid=code.uuid[:10])
        self.assertEquals(loaded_code.uuid, code.uuid)

        with self.assertRaises(NotExistent):
            load_code('non-existent-uuid')

    def test_load_computer(self):
        """Test the functionality of load_group."""
        computer = self.computer

        # Load through label
        loaded_computer = load_computer(computer.label)
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through uuid
        loaded_computer = load_computer(computer.uuid)
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through pk
        loaded_computer = load_computer(computer.pk)
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through label explicitly
        loaded_computer = load_computer(label=computer.label)
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through uuid explicitly
        loaded_computer = load_computer(uuid=computer.uuid)
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through pk explicitly
        loaded_computer = load_computer(pk=computer.pk)
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through partial uuid without a dash
        loaded_computer = load_computer(uuid=computer.uuid[:8])
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        # Load through partial uuid including a dash
        loaded_computer = load_computer(uuid=computer.uuid[:10])
        self.assertEquals(loaded_computer.uuid, computer.uuid)

        with self.assertRaises(NotExistent):
            load_computer('non-existent-uuid')

    def test_load_group(self):
        """Test the functionality of load_group."""
        name = 'groupie'
        group = Group(label=name).store()

        # Load through label
        loaded_group = load_group(group.label)
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through uuid
        loaded_group = load_group(group.uuid)
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through pk
        loaded_group = load_group(group.pk)
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through label explicitly
        loaded_group = load_group(label=group.label)
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through uuid explicitly
        loaded_group = load_group(uuid=group.uuid)
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through pk explicitly
        loaded_group = load_group(pk=group.pk)
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through partial uuid without a dash
        loaded_group = load_group(uuid=group.uuid[:8])
        self.assertEquals(loaded_group.uuid, group.uuid)

        # Load through partial uuid including a dash
        loaded_group = load_group(uuid=group.uuid[:10])
        self.assertEquals(loaded_group.uuid, group.uuid)

        with self.assertRaises(NotExistent):
            load_group('non-existent-uuid')

    def test_load_node(self):
        """Test the functionality of load_node."""
        node = Data().store()

        # Load through uuid
        loaded_node = load_node(node.uuid)
        self.assertIsInstance(loaded_node, Node)
        self.assertEquals(loaded_node.uuid, node.uuid)

        # Load through pk
        loaded_node = load_node(node.pk)
        self.assertIsInstance(loaded_node, Node)
        self.assertEquals(loaded_node.uuid, node.uuid)

        # Load through uuid explicitly
        loaded_node = load_node(uuid=node.uuid)
        self.assertIsInstance(loaded_node, Node)
        self.assertEquals(loaded_node.uuid, node.uuid)

        # Load through pk explicitly
        loaded_node = load_node(pk=node.pk)
        self.assertIsInstance(loaded_node, Node)
        self.assertEquals(loaded_node.uuid, node.uuid)

        # Load through partial uuid without a dash
        loaded_node = load_node(uuid=node.uuid[:8])
        self.assertIsInstance(loaded_node, Node)
        self.assertEquals(loaded_node.uuid, node.uuid)

        # Load through partial uuid including a dashs
        loaded_node = load_node(uuid=node.uuid[:10])
        self.assertIsInstance(loaded_node, Node)
        self.assertEquals(loaded_node.uuid, node.uuid)

        with self.assertRaises(NotExistent):
            load_group('non-existent-uuid')



# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################
"""Module that contains the class definitions necessary to offer support for queries to Materials Project."""
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import pytest

from aiida.backends.testbase import AiidaTestCase
from aiida.plugins import DbImporterFactory


def run_materialsproject_api_tests():
    from aiida.settings import PROFILE_CONF
    return PROFILE_CONF.get('run_materialsproject_api_tests', False)


class TestMaterialsProject(AiidaTestCase):
    """
    Contains the tests to verify the functionality of the Materials Project importer
    functions.
    """

    import unittest

    @unittest.skipIf(not run_materialsproject_api_tests(), "MaterialsProject API tests not enabled")
    def test_invalid_api_key(self):  # pylint: disable=no-self-use
        """
        Test if Materials Project rejects an invalid API key and that we catch the error.
        Please enable the test in the profile configurator.
        """
        importer_class = DbImporterFactory('materialsproject')
        importer_parameters = {'api_key': 'thisisawrongkey'}

        with pytest.raises(Exception):
            importer_class(**importer_parameters)



# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################
"""Tests for process spec ports."""
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from aiida.backends.testbase import AiidaTestCase
from aiida.engine.processes.ports import InputPort, PortNamespace


class TestInputPort(AiidaTestCase):
    """Tests for the `InputPort` class."""

    def test_with_non_db(self):
        """Test the functionality of the `non_db` attribute upon construction and setting."""

        # When not specifying, it should get the default value and `non_db_explicitly_set` should be `False`
        port = InputPort('port')
        self.assertEqual(port.non_db, False)
        self.assertEqual(port.non_db_explicitly_set, False)

        # Using the setter to change the value should toggle both properties
        port.non_db = True
        self.assertEqual(port.non_db, True)
        self.assertEqual(port.non_db_explicitly_set, True)

        # Explicitly setting to `False` upon construction
        port = InputPort('port', non_db=False)
        self.assertEqual(port.non_db, False)
        self.assertEqual(port.non_db_explicitly_set, True)

        # Explicitly setting to `True` upon construction
        port = InputPort('port', non_db=True)
        self.assertEqual(port.non_db, True)
        self.assertEqual(port.non_db_explicitly_set, True)


class TestPortNamespace(AiidaTestCase):
    """Tests for the `PortNamespace` class."""

    def test_with_non_db(self):
        """Ports inserted to a `PortNamespace` should inherit the `non_db` attribute if not explicitly set."""
        namespace_non_db = True
        port_namespace = PortNamespace('namespace', non_db=namespace_non_db)

        # When explicitly set upon port construction, value should not be inherited even when different
        port = InputPort('storable', non_db=False)
        port_namespace['storable'] = port
        self.assertEqual(port.non_db, False)

        port = InputPort('not_storable', non_db=True)
        port_namespace['not_storable'] = port
        self.assertEqual(port.non_db, True)

        # If not explicitly defined, it should inherit from parent namespace
        port = InputPort('not_storable')
        port_namespace['not_storable'] = port
        self.assertEqual(port.non_db, namespace_non_db)

    def test_validate_port_name(self):
        """This test will ensure that illegal port names will raise a `ValueError` when trying to add it."""
        port = InputPort('port')
        port_namespace = PortNamespace('namespace')

        illegal_port_names = [
            'two__underscores',
            'three___underscores',
            '_leading_underscore',
            'trailing_underscore_',
            'non_numeric_%',
            'including.period',
            'disallowed👻unicodecharacters',
            'white space',
            'das-hes',
        ]

        for port_name in illegal_port_names:
            with self.assertRaises(ValueError):
                port_namespace[port_name] = port



# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

from aiida.common import exceptions
from aiida.orm import Int
from aiida.parsers.parser import Parser


class ArithmeticAddParser(Parser):

    def parse(self, **kwargs):
        """Parse the contents of the output files retrieved in the `FolderData`."""
        try:
            output_folder = self.retrieved
        except exceptions.NotExistent:
            return self.exit_codes.ERROR_NO_RETRIEVED_FOLDER

        try:
            with output_folder.open(self.node.get_attribute('output_filename'), 'r') as handle:
                result = self.parse_stdout(handle)
        except (OSError, IOError):
            return self.exit_codes.ERROR_READING_OUTPUT_FILE

        if result is None:
            return self.exit_codes.ERROR_INVALID_OUTPUT

        self.out('sum', Int(result))

    @staticmethod
    def parse_stdout(filelike):
        """
        Parse the sum from the output of the ArithmeticAddcalculation written to standard out

        :param filelike: filelike object containing the output
        :returns: the sum
        """
        try:
            result = int(filelike.read())
        except ValueError:
            result = None

        return result



# -*- coding: utf-8 -*-
###########################################################################
# Copyright (c), The AiiDA team. All rights reserved.                     #
# This file is part of the AiiDA code.                                    #
#                                                                         #
# The code is hosted on GitHub at https://github.com/aiidateam/aiida_core #
# For further information on the license, see the LICENSE.txt file        #
# For further information please visit http://www.aiida.net               #
###########################################################################
"""Module to define the custom click type for code."""
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import click


class ConfigOptionParamType(click.types.StringParamType):
    """ParamType for configuration options."""

    name = 'config option'

    def convert(self, value, param, ctx):
        from aiida.manage.configuration.options import get_option, get_option_names

        if value not in get_option_names():
            raise click.BadParameter('{} is not a valid configuration option'.format(value))

        return get_option(value)

    def complete(self, ctx, incomplete):  # pylint: disable=unused-argument,no-self-use
        """
        Return possible completions based on an incomplete value

        :returns: list of tuples of valid entry points (matching incomplete) and a description
        """
        from aiida.manage.configuration.options import get_option_names

        return [(option_name, '') for option_name in get_option_names() if option_name.startswith(incomplete)]
